# 3xx 兼容层

from ncatbot.plugin_system import NcatBotEvent as Event
from ncatbot.plugin_system import NcatBotPlugin as BasePlugin
from ncatbot.plugin.compatible_enrollment import CompatibleEnrollment


__all__ = [
    'Event',
    'BasePlugin',
    'CompatibleEnrollment',
]

