"""Constants used throughout the NcatBot CLI."""

# Package management
PYPI_SOURCE = "https://mirrors.aliyun.com/pypi/simple/"

# URLs
PLUGIN_INDEX_URL = (
    "https://raw.githubusercontent.com/ncatbot/NcatBot-Plugins/main/index.json"
)
